#ifndef _FIXEDLENGTH_CPP_
#define _FIXEDLENGTH_CPP_

#include <iostream.h>

#include <reference.h>
#include <table.h>
#include <event.h>
#include <part.h>
#include <track.h>
#include <song.h>
#include <str.h>
#include <prMainEditor.h>

#include "fixedLength.h"

extern Table * selectioN;
extern Table * selectionMemory;
extern PrMainEditor * mainEditor;

#define FIXEDLENGTH_CONTEXT Addon::TREAT_SELECTION + Addon::TREAT_TRACK + Addon::TREAT_PART + Addon::TREAT_SONG

extern "C" {
  Addon * fixedLength(Element * target) { return new FixedLength(target); }
  const char  * fixedLength_name() { return "set fixed length"; }
  const char  * fixedLength_category() { return "quantize"; }
  int     fixedLength_context() { return FIXEDLENGTH_CONTEXT; }
}


FixedLength::FixedLength(Element * target) : Addon("quantize", target, TO_BE_UNDONE, FIXEDLENGTH_CONTEXT), context(0), count(0) { run(); }

FixedLength::~FixedLength() {

}

void setLength(Event * ev, int grid) {
  ev->setDuration(grid);
}

void FixedLength::selection(Table * selection) {
  context = Addon::TREAT_SELECTION;
  int grid = mainEditor->grid();
  int sz = selection->size();
  len = new long[sz];
  for (Reference * r = (Reference*)selection->first(); r != 0; r = (Reference*) Element::next(r)) {
    Event * ev = (Event*) r->getValue();
    len[count++] = ev->duration();
    setLength(ev,grid);
  }
}

void FixedLength::part(Part * part) {
  context = Addon::TREAT_PART;
  int grid = mainEditor->grid();
  int sz = part->size();
  len = new long[sz];
  for (Event * ev = (Event*)part->first(); ev != 0; ev = (Event*) Element::next(ev)) {
    len[count++] = ev->duration();
    setLength(ev,grid);
  }
}

void FixedLength::track(Track * track) {
  context = Addon::TREAT_TRACK;
  int grid = mainEditor->grid();
  int sz = track->size();
  int totalsz = 0;
  for (int i=0; i<sz; i++) {
    Part * pt = (Part*) track->get(i);
    totalsz += pt->size();
  }
  len = new long[totalsz];
  for (int i=0; i<sz; i++) {
    Part * pt = (Part*) track->get(i);
    for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
      len[count++] = ev->duration();
      setLength(ev,grid);
    }
  }
}

void FixedLength::song(Song * song) {
  context = Addon::TREAT_SONG;
  int grid = mainEditor->grid();
  int totalsz = 0;
  int szs = song->size();
  for (int i=0; i<szs; i++) {
    Track * track = (Track*) song->get(i);
    int sz = track->size();
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      totalsz += pt->size();
    }
  }
  len = new long[totalsz];
  for (int i=0; i<szs; i++) {
    Track * track = (Track*) song->get(i);
    int sz = track->size();
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	len[count++] = ev->duration();
	setLength(ev,grid);
      }
    }
  }
}

void FixedLength::undo() {
  long foo2 = 0;
  int mem = 0;
  if (context == Addon::TREAT_SELECTION) {
    Table * selection = (Table*) _target;
    for (Reference * r = (Reference*)selection->first(); r != 0; r = (Reference*) Element::next(r)) {
      Event * ev = (Event*) r->getValue();
      foo2 = ev->duration();
      ev->setDuration(len[mem]);
      len[mem++] = foo2;
    }
  } else if (context == Addon::TREAT_PART) {
    Part * part = (Part*) _target;
    for (Event * ev = (Event*)part->first(); ev != 0; ev = (Event*) Element::next(ev)) {
      foo2 = ev->duration();
      ev->setDuration(len[mem]);
      len[mem++] = foo2;
    }
  } else if (context == Addon::TREAT_TRACK) {
    Track * track = (Track*) _target;
    int sz = track->size();
    int totalsz = 0;
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      totalsz += pt->size();
    }
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	foo2 = ev->duration();
	ev->setDuration(len[mem]);
	len[mem++] = foo2;
      }
    }
  } else if (context == Addon::TREAT_SONG) {
    Song * song = (Song*) _target;
    int totalsz = 0;
    int szs = song->size();
    for (int i=0; i<szs; i++) {
      Track * track = (Track*) song->get(i);
      int sz = track->size();
      for (int i=0; i<sz; i++) {
	Part * pt = (Part*) track->get(i);
	totalsz += pt->size();
      }
    }
    for (int i=0; i<szs; i++) {
      Track * track = (Track*) song->get(i);
      int sz = track->size();
      for (int i=0; i<sz; i++) {
	Part * pt = (Part*) track->get(i);
	for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	  foo2 = ev->duration();
	  ev->setDuration(len[mem]);
	  len[mem++] = foo2;
	}
      }
    }
  }
}

void FixedLength::redo() {
  long foo2 = 0;
  int mem = 0;
  if (context == Addon::TREAT_SELECTION) {
    Table * selection = (Table*) _target;
    for (Reference * r = (Reference*)selection->first(); r != 0; r = (Reference*) Element::next(r)) {
      Event * ev = (Event*) r->getValue();
      foo2 = ev->duration();
      ev->setDuration(len[mem]);
      len[mem++] = foo2;
    }
  } else if (context == Addon::TREAT_PART) {
    Part * part = (Part*) _target;
    for (Event * ev = (Event*)part->first(); ev != 0; ev = (Event*) Element::next(ev)) {
      foo2 = ev->duration();
      ev->setDuration(len[mem]);
      len[mem++] = foo2;
    }
  } else if (context == Addon::TREAT_TRACK) {
    Track * track = (Track*) _target;
    int sz = track->size();
    int totalsz = 0;
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      totalsz += pt->size();
    }
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	foo2 = ev->duration();
	ev->setDuration(len[mem]);
	len[mem++] = foo2;
      }
    }
  } else if (context == Addon::TREAT_SONG) {
    Song * song = (Song*) _target;
    int totalsz = 0;
    int szs = song->size();
    for (int i=0; i<szs; i++) {
      Track * track = (Track*) song->get(i);
      int sz = track->size();
      for (int i=0; i<sz; i++) {
	Part * pt = (Part*) track->get(i);
	totalsz += pt->size();
      }
    }
    for (int i=0; i<szs; i++) {
      Track * track = (Track*) song->get(i);
      int sz = track->size();
      for (int i=0; i<sz; i++) {
	Part * pt = (Part*) track->get(i);
	for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	  foo2 = ev->duration();
	  ev->setDuration(len[mem]);
	  len[mem++] = foo2;
	}
      }
    }
  }
}






#endif
