/***************************************************************************
                          mididriver.h  -  description
                             -------------------
    begin                : Sun Jun 9 2002
    copyright            : (C) 2002 by Morten Hustveit
    email                : mortehu@ifi.uio.no
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MIDIDRIVER_H
#define MIDIDRIVER_H


/**
  *@author Morten Hustveit
  */

#include <sys/poll.h>

#include <qptrvector.h>
#include <qvaluelist.h>

#include "configuration.h"

class MIDIDriver
{
public:

  static void init();
  static void reconfigure(const Configuration* config);
  static void close();

  static void put(int port, int data);
  static void putRT(int port, int data);
  static void broadcast(int data);
  static void broadcastRT(int data);
  static void clockStep();
  static void drain();

  static void setBPM(double bpm);
  static double bpm();

  static int get(int& port);

  static int portCount();

  static Configuration::PortInfos portInfos();

  class Port
  {
  public:

    Port() { status = 0; }
    virtual ~Port() { };
    virtual void putRT(int data) = 0;
    virtual void put(int data) = 0;
    virtual void clockStep() = 0;
    virtual void drain() = 0;
    virtual bool empty() = 0;
    virtual QValueList<struct pollfd> pollfds() = 0;
    virtual int  get() = 0;
    virtual bool isOpen() const = 0;

    int status;
  };

private:

  static QValueList<unsigned char> _buffer;
  static QPtrVector<Port> _ports;
  static int _reconfPipe[2]; // To release a blocking poll or select
};

#endif

