/***************************************************************************
                          sanetrackerview.cpp  -  description
                             -------------------
    begin                : Sat Jun  8 15:33:59 CEST 2002
    copyright            : (C) 2002 by Morten Hustveit
    email                : mortehu@ifi.uio.no
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qsplitter.h>
#include <qlayout.h>

// application specific includes
#include "sanetrackerview.h"
#include "sanetrackerdoc.h"
#include "sanetracker.h"
#include "scheduler.h"
#include "sequence.h"
#include "pattern.h"
#include "piano.h"
#include "configuration.h"

SanetrackerView::SanetrackerView(QWidget *parent, const Configuration* config, Scheduler& scheduler)
  : QWidget(parent, 0),
    scheduler(scheduler)
{
  setBackgroundMode(PaletteBase);

  setFocusPolicy(QWidget::StrongFocus);

  splitter = new QSplitter(this);
  sequence = new Sequence(splitter, getDocument(), scheduler);
  pattern = new Pattern(splitter, *getDocument(), config, scheduler);
  piano = new Piano(this);

  vLayout = new QVBoxLayout(this);
  vLayout->addWidget(splitter);
  vLayout->addWidget(piano);

  sequence->setActive();
  pattern->setActive(false);

  splitter->setOrientation(QSplitter::Vertical);

  connect(sequence, SIGNAL(selectionChanged(uint, uint, uint)), pattern, SLOT(edit(uint, uint, uint)));
  connect(pattern, SIGNAL(activated()), this, SLOT(patternActivated()));
  connect(sequence, SIGNAL(activated()), this, SLOT(sequenceActivated()));
  connect(pattern, SIGNAL(nextPattern()), sequence, SLOT(slotNextPattern()));
  connect(pattern, SIGNAL(prevPattern()), sequence, SLOT(slotPrevPattern()));

  // Set sequence view to take 30% of the available space
  QValueList<int> sizes = splitter->sizes();
  int total = sizes[0] + sizes[1];
  sizes[0] = (int) (0.3 * total);
  sizes[1] = total - sizes[0];
  splitter->setSizes(sizes);

  this->config = config;
}

void SanetrackerView::positionChanged()
{
  sequence->positionChanged();
}

void SanetrackerView::reconfigure()
{
  sequence->reconfigure();
}

void SanetrackerView::edit(uint device, uint pattern, uint position)
{
  this->pattern->edit(device, pattern, position);
}

void SanetrackerView::sequenceActivated()
{
  sequence->setActive();
  pattern->setActive(false);
}

void SanetrackerView::patternActivated()
{
  pattern->setActive();
  sequence->setActive(false);
}

bool SanetrackerView::event(QEvent* event)
{
  if(event->type() == QEvent::KeyPress)
  {
    QKeyEvent* keyEvent = (QKeyEvent*) event;

    if(keyEvent->key() != Key_Tab && keyEvent->key() != Key_BackTab)
      if(QWidget::event(event))
        return true;

    if(keyEvent->key() == Key_Tab || keyEvent->key() == Key_BackTab)
    {
      if(pattern->active())
        sequenceActivated();
      else // sequence->active()
        patternActivated();

      return true;
    }
    else if(keyEvent->key() == Key_Space)
    {
      if(!scheduler.isPlaying())
        scheduler.play(sequence->getCurrentCol(), 0);
      else // scheduler.isPlaying()
        scheduler.stopPlayback();

      return true;
    }

    if(pattern->active())
    {
      pattern->keyPressed(keyEvent);
    }
    else // sequence->active()
    {
      sequence->keyPressed(keyEvent);
    }

    return keyEvent->isAccepted();
  }
  else if(event->type() == QEvent::FocusOut)
  {
    if(sequence->active())
    {
      lastFocus = 0;
      sequence->setActive(false);
    }
    else // pattern->activated()
    {
      lastFocus = 1;
      sequence->setActive(false);
    }

    return true;
  }
  else if(event->type() == QEvent::FocusIn)
  {
    if(lastFocus == 0)
    {
      sequence->setActive();
    }
    else // lastFocus == 1
    {
      pattern->setActive();
    }

    return true;
  }
  else
  {
    return QWidget::event(event);
  }
}

SanetrackerDoc *SanetrackerView::getDocument() const
{
  return reinterpret_cast<SanetrackerApp*>(parentWidget())->getDocument();
}

SanetrackerView::~SanetrackerView()
{
  delete splitter;
  delete pattern;
  delete sequence;
  delete piano;
}

